#include <SDL2/SDL_assert.h>
#include <SDL2/SDL_events.h>

#include <basic_widgets/w/base/widget.hpp>
#include <basic_widgets/w/feat/keyboard_handler.hpp>
#include <basic_widgets/w/feat/mouse_handler.hpp>

using namespace bwidgets;

auto Widget::handle_event(const SDL_Event& ev) -> Widget*
{
    if (auto* handler = dynamic_cast<KeyboardHandler*>(this); handler) {
        switch (ev.type) {
            case SDL_KEYDOWN:
            case SDL_KEYUP:
                handler->handle_keyboard(ev.key);
                break;
            case SDL_TEXTINPUT:
                handler->handle_keyboard(ev.text);
                break;
        }
    }

    if (auto* handler = dynamic_cast<MouseHandler*>(this); handler) {
        switch (ev.type) {
            case SDL_MOUSEBUTTONDOWN:
            case SDL_MOUSEBUTTONUP:
                handler->handle_mouse(ev.button, _viewport);
                break;
            case SDL_MOUSEMOTION:
                handler->handle_mouse(ev.motion, _viewport);
                break;
        }
    }

    return this;
}

auto Widget::render() -> Widget*
{
    if (_renderer == nullptr) return this;

#ifndef _NDEBUG
    _renderer
      ->draw_color({
  // NOLINTNEXTLINE(readability-magic-numbers)
        {0, 255, 0, SDL_ALPHA_TRANSPARENT}
    })
      ->draw_rect(nullptr);
#endif

    _renderer->viewport(_viewport);
    _handle_rendering();

    return this;
}

auto Widget::renderer(Renderer* r) -> Widget*
{
    if (r != _renderer) {
        _handle_renderer_change(r);
        _renderer = r;
    }
    return this;
}

auto Widget::viewport(const SDL_Rect& vp) -> Widget*
{
    _handle_geometry_change(vp);
    _viewport = vp;
    return this;
}

auto Widget::viewport() const -> const SDL_Rect&
{
    return _viewport;
}
